#app{font-family:Avenir,Helvetica,Arial,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.main-layout[data-v-6a3e4a68]{height:100vh}.sider-bar[data-v-6a3e4a68]{background-color:#515a6e}.sider-bar-divider[data-v-6a3e4a68]{height:1px;margin:0;background:#6c6c6c}.logo[data-v-6a3e4a68]{height:38px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center}.logo span[data-v-6a3e4a68]{color:#fff;font-size:25px;font-weight:bolder;font-style:italic}.logo img[data-v-6a3e4a68]{width:32px}.main-header[data-v-6a3e4a68]{height:38px;line-height:38px;padding:0;margin:0;background-color:#0fccbf}.main-footer[data-v-6a3e4a68]{height:28px;line-height:28px;padding:0;background-color:#0fccbf}.main-footer-copyright[data-v-6a3e4a68]{font-size:12px;margin-left:15px;color:#f8f8f9}.main-footer-status[data-v-6a3e4a68]{float:right;font-size:11px;margin-right:10px}.collapsed-menu span[data-v-6a3e4a68]{width:0;-webkit-transition:width .2s ease;transition:width .2s ease}.menu-item span[data-v-6a3e4a68]{display:inline-block;overflow:hidden;width:69px;text-overflow:ellipsis;white-space:nowrap;vertical-align:bottom;-webkit-transition:width .2s ease .2s;transition:width .2s ease .2s}.main-container[data-v-6a3e4a68]{padding:5px;height:calc(100vh - 66px)}.flow-list{height:calc(100vh - 166px);overflow-y:auto}.data-table td div,.data-table th div{padding-left:5px;padding-right:5px}.flow-detail{height:calc(100vh - 218px)}.button-bar[data-v-6b5ac708]{margin-bottom:5px;height:48px}.inspector-left[data-v-6b5ac708]{margin-right:5px}.inspector-right[data-v-6b5ac708]{margin-left:5px}.inline{display:inline}.data-list{height:calc(100vh - 166px);overflow-y:auto}.button-bar{margin-top:5px}.data-detail{height:calc(100vh - 253px)}.button-bar[data-v-685669b0]{margin-bottom:5px;height:48px}.container-left[data-v-685669b0]{margin-right:5px}.container-right[data-v-685669b0]{margin-left:5px}.plugin-frame{position:relative;height:100%;width:100%}